//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

// TotalPassword.cpp: implementation of the CTotalPassword class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TotalPassword.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTotalPassword::CTotalPassword()
{
	m_pszPassword = new TCHAR[1024];
	memset(m_pszPassword, 0, 1024 * sizeof(TCHAR));

	m_nSeed = 0;
}

CTotalPassword::~CTotalPassword()
{
	if(m_pszPassword)
		delete [] m_pszPassword;
}

TCHAR* CTotalPassword::GetPassword(TCHAR* pszPassword)
{
	int nLen = lstrlen(pszPassword);
	if(nLen)
	{
		BYTE* pPassword = new BYTE[nLen + 1];
		memset(pPassword, 0, nLen + 1);

		for(int i=0; i<nLen; i++)
			pPassword[i] = (BYTE)pszPassword[i];

		if(Decrypt(pPassword))
		{
			delete [] pPassword;

			if(m_pszPassword)
				return m_pszPassword;
		}
	}

	return NULL;
}

int CTotalPassword::Hex2Byte(BYTE* pPassword)
{
	int n = 0;

	TCHAR *p = (TCHAR*)pPassword;
	
	for(int i = 0; *p; i++, p += 2) 
	{
		if(_stscanf(p, _T("%02X"), &n) != 1)
			break;

		pPassword[i] = n;
	}

	return i;
}

int CTotalPassword::MakeRandom(UINT nMax) 
{
	m_nSeed = (m_nSeed * 0x8088405) + 1;
	return (int)(((LONGLONG)m_nSeed * nMax) >> 32);
}

UINT CTotalPassword::Roll(UINT n1, UINT n2) 
{
	return((n1 << n2) | (n1 >> (8 - n2)));
}

BOOL CTotalPassword::Decrypt(BYTE* pPassword)
{
	int nLen = Hex2Byte(pPassword);

	nLen -= 4;
	if(nLen < 0)
		return FALSE;

	m_nSeed = 849521;

	for(int i=0; i<nLen; i++)
		pPassword[i] = Roll(pPassword[i], MakeRandom(8));

	int nFirst = 0;
	int nLast = 0;
	UCHAR u = 0x0;
	m_nSeed = 12345;
	for(i=0; i<256; i++)
	{
		nFirst = MakeRandom(nLen);
		nLast = MakeRandom(nLen);
		u = pPassword[nFirst];
		pPassword[nFirst] = pPassword[nLast];
		pPassword[nLast] = u;
	}

	m_nSeed = 42340;
	for(i=0; i<nLen; i++) 
		pPassword[i] ^= MakeRandom(256);

	m_nSeed = 54321;
	for(i=0; i<nLen; i++)
		pPassword[i] -= MakeRandom(256);

	pPassword[nLen] = '\0';
	for(i=0; i<nLen; i++)
		m_pszPassword[i] = pPassword[i];

	return TRUE;
}
